<?php

/**
 * Clase para consumir API Rest
 * Las operaciones soportadas son:
 *
 *    - POST        : Agregar
 *    - GET        : Consultar
 *    - DELETE    : Eliminar
 *    - PUT        : Actualizar
 *    - PATCH        : Actualizar por parte
 *
 */
class API
{
    /**
     * Enviar parámetros a un servidor a través del protocolo HTTP (POST).
     * Se utiliza para agregar datos en una Api REST
     *
     * @param string $URL URL recurso, ejemplo: http://website.com/recurso
     * @param string $TOKEN token de autenticación
     * @param array $ARRAY parámetros a envíar
     * @return JSON
     */
    static function POST($URL, $ARRAY, $TOKEN = NULL)
    {
        $datapost = $ARRAY;
        if ($TOKEN != NULL)
            $headers = array('Authorization: ' . $TOKEN);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        curl_setopt($ch, CURLOPT_URL, $URL);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $datapost);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);
        curl_setopt($ch, CURLOPT_TIMEOUT, 100);

        if ($TOKEN != NULL)
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $response = curl_exec($ch);
        if (curl_errno($ch)) {
            return json_decode(curl_error($ch));
        }
        curl_close($ch);
        //var_dump(curl_getinfo($ch));
        return $response;
    }

    static function POST_SERVER($URL, $ARRAY, $TOKEN = NULL)
    {

        if ($TOKEN != NULL)
            $headers = array('Authorization: ' . $TOKEN);
        $encoded = '';
        foreach ($ARRAY as $name => $value) {
            $encoded .= urlencode($name) . '=' . urlencode($value) . '&';
        }
// chop off the last ampersand
        $encoded = substr($encoded, 0, strlen($encoded) - 1);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        curl_setopt($ch, CURLOPT_URL, $URL);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $encoded);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);
        curl_setopt($ch, CURLOPT_TIMEOUT, 100);

        if ($TOKEN != NULL)
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $response = curl_exec($ch);
        curl_close($ch);
        return $response;
    }

    /**
     * Consultar a un servidor a través del protocolo HTTP (GET).
     * Se utiliza para consultar recursos en una Api REST
     *
     * @param string $URL URL recurso, ejemplo: http://website.com/recurso/(id) no obligatorio
     * @param string $TOKEN token de autenticación
     * @return JSON
     */
    static function GET($URL, $TOKEN = NULL)
    {
        if ($TOKEN != NULL)
            $headers = array('Authorization: ' . $TOKEN);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        curl_setopt($ch, CURLOPT_URL, $URL);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 100);
        if ($TOKEN != NULL)
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $response = curl_exec($ch);
        curl_close($ch);
        return $response;
    }

    /**
     * Consultar a un servidor a través del protocolo HTTP (DELETE).
     * Se utiliza para eliminar recursos en una Api REST
     *
     * @param string $URL URL recurso, ejemplo: http://website.com/recurso/id
     * @param string $TOKEN token de autenticación
     * @return JSON
     */
    static function DELETE($URL, $TOKEN)
    {
        $headers = array('Authorization: ' . $TOKEN);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        curl_setopt($ch, CURLOPT_URL, $URL);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "DELETE");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 20);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        $response = curl_exec($ch);
        curl_close($ch);
        return $response;
    }

    /**
     * Enviar parámetros a un servidor a través del protocolo HTTP (PUT).
     * Se utiliza para editar un recurso en una Api REST
     *
     * @param string $URL URL recurso, ejemplo: http://website.com/recurso/id
     * @param string $TOKEN token de autenticación
     * @param array $ARRAY parámetros a envíar
     * @return JSON
     */
    static function PUT($URL, $TOKEN, $ARRAY)
    {
        $datapost = '';
        foreach ($ARRAY as $key => $value) {
            $datapost .= $key . "=" . $value . "&";
        }
        $headers = array('Authorization: ' . $TOKEN);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $URL);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $datapost);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);
        curl_setopt($ch, CURLOPT_TIMEOUT, 20);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        $response = curl_exec($ch);
        curl_close($ch);
        return $response;
    }

    /**
     * Convertir JSON a un ARRAY
     *
     * @param json $json Formato JSON
     * @return ARRAY
     */
    static function JSON_TO_ARRAY($json)
    {
        return json_decode($json, true);
    }

    static function rest_helper($url, $params = null, $verb = 'GET', $format = 'json')
    {
        $cparams = array(
            'http' => array(
                'method' => $verb,
                'ignore_errors' => true
            )
        );
        if ($params !== null) {
            $params = http_build_query($params);
            if ($verb == 'POST') {
                $cparams['http']['content'] = $params;
            } else {
                $url .= '?' . $params;
            }
        }

        $context = stream_context_create($cparams);
        $fp = fopen($url, 'rb', false, $context);
        if (!$fp) {
            $res = false;
        } else {
            // If you're trying to troubleshoot problems, try uncommenting the
            // next two lines; it will show you the HTTP response headers across
            // all the redirects:
            // $meta = stream_get_meta_data($fp);
            // var_dump($meta['wrapper_data']);
            $res = stream_get_contents($fp);
        }

        if ($res === false) {
            throw new Exception("$verb $url failed: $php_errormsg");
        }

        switch ($format) {
            case 'json':
                $r = json_decode($res);
                if ($r === null) {
                    throw new Exception("failed to decode $res as json");
                }
                return $res;

            case 'xml':
                $r = simplexml_load_string($res);
                if ($r === null) {
                    throw new Exception("failed to decode $res as xml");
                }
                return $r;
        }
        return $res;
    }


    static function multiCurl($res)
    {

        if (count($res) <= 0) return False;

        $handles = array();
        // add curl options to each handle
        foreach ($res as $k => $row) {
            $ch{$k} = curl_init();

            $TOKEN = $row['token'];
            if ($TOKEN != NULL)
                $headers = array('Authorization: ' . $TOKEN);

            if ($TOKEN != NULL){
                $options = array(
                    CURLOPT_HEADER => 0,
                    CURLOPT_RETURNTRANSFER => 1,
                    CURLOPT_POSTFIELDS => $row['post'],
                    CURLOPT_HTTPHEADER => $headers
                );
            } else {
                $options = array(
                    CURLOPT_HEADER => 0,
                    CURLOPT_RETURNTRANSFER => 1,
                    CURLOPT_POSTFIELDS => $row['post'],
                    CURLOPT_HTTPHEADER => $headers
                );
            }
            $options[CURLOPT_URL] = $row['url'];

            curl_setopt_array($ch{$k}, $options);
            //var_dump($opt);
            $handles[$k] = $ch{$k};
        }

        $mh = curl_multi_init();

        // add handles
        foreach ($handles as $k => $handle) {
            $err = curl_multi_add_handle($mh, $handle);
        }

        $running_handles = null;

        do {
            curl_multi_exec($mh, $running_handles);
            curl_multi_select($mh);
        } while ($running_handles > 0);

        foreach ($res as $k => $row) {
            $res[$k]['error'] = curl_error($handles[$k]);
            if (!empty($res[$k]['error']))
                $res[$k]['data'] = '';
            else
                $res[$k]['data'] = curl_multi_getcontent($handles[$k]);  // get results

            // close current handler
            curl_multi_remove_handle($mh, $handles[$k]);
        }
        curl_multi_close($mh);
        return $res; // return response
    }


}

?>