<?php
ini_set('memory_limit', '-1');
require_once 'ConnectionDB.php';

class Articulos
{
    const TABLE_NAME = "dex_articulos";
    const ID_ARTICULO = "id_articulo";
    const DOC_ID = "doc_id";
    const ART_STATUS = "art_status";
    const ART_NUM = "art_num";
    const ART_TITULO = "art_titulo";
    const ART_DESC = "art_desc";
    const ART_DESC_NO_HTML = "art_desc_no_html";
    const ART_EVOLUCION = "art_evolucion";
    const ART_EVOLUCION_NO_HTML = "art_evolucion_no_html";
    const ART_SORT_POSITION = "art_sort_position";
    const FECHA_MODIFICACION = "fecha_modificacion";
    const IDINTERNO = "idinterno";
    const NUMDCT = "numdct";
    const NUMDCTRO = "numdctro";
    const FECHADCT = "fechadct";

    /*STATUS USER*/
    const SUCCESSFUL_CREATION = 1;
    const FAILED_CREATION = 2;
    const ERROR_DATABASE = 3;
    const AUS_AUTH_API_KEY = 4;//ausencia api
    const NOT_AUTH_API_KEY = 5;//no autorizada
    const INCORRECT_URL = 6;
    const UNKNOWN_FAULT = 7;
    private static $encryption_key = '5166546A576E5A72';

    public static function setPostRequest($request)
    {

        if ($request[0] == 'allbyid') {
            return Articulos::getAllById();
        } else if ($request[0] == 'getartbyid') {
            return Articulos::getArtById();
        } else if ($request[0] == 'getartbysort') {
            return Articulos::getArtBySort();
        } else if ($request[0] == 'download_all') {
            return Articulos::downloadAll();
        } else {
            throw new ExceptionApi(self::INCORRECT_URL, "Url mal formada", 400);
        }

    }

    public static function setGetRequest($request)
    {
        if ($request[0] == 'all') {

        } else {
            throw new ExceptionApi(self::INCORRECT_URL, "Url mal formada", 400);
        }

    }


    private static function downloadAll()
    {

        $body = file_get_contents('php://input');
        $docObject = json_decode($body);
        // Validar campos
        User::authorizationFromApp();
        if ($docObject !== null && ValidComponent::issetField($docObject->doc_id, 900)) {
            $doc_id = $docObject->doc_id;
            $filename = "autogenerate_json_file_" . $docObject->doc_id;
            $resulArticulos = self::getArticulosByDocumIDWithoutLimit($doc_id);
            //$resulArticulos = self::getArticulosByDocumID($doc_id, 0);
            $concordancia = Correspondencia::encryptDataCorrespondencia(Correspondencia::getInfoCorrespondencia($doc_id));

            $content = [
                "status" => self::SUCCESSFUL_CREATION,
                "concordancia" => $concordancia,
                "data" => self::encryptDataArticulos($resulArticulos)
            ];
            $fsize = strlen(json_encode($content, JSON_NUMERIC_CHECK));
            //$fsize = mb_strlen(json_encode($content, JSON_PRETTY_PRINT), '8bit');;
            header('Content-Description: File Transfer');
            header('Content-Type: application/json');//important!
            header("Content-Disposition: attachment; filename=\"" . $filename . "\"");
            //header("Content-length: $fsize");
            header("Content-Range: 0-" . ($fsize - 1) . "/" . $fsize);
            header("Cache-control: private"); //use this to open files directly

            return $content;

        } else {
            throw new ExceptionApi(self::INCORRECT_URL, "Parametros incorrectos", 400);
        }


    }

    private static function getArtBySort()
    {
        $body = file_get_contents('php://input');
        $docObject = json_decode($body);
        // Validar campos
        User::authorizationFromApp();
        if ($docObject !== null
            && ValidComponent::issetField($docObject->doc_id, 900)
            && ValidComponent::issetField($docObject->art_sort_position, 901)
            && ValidComponent::issetField($docObject->action, 902)) {
            $doc_id = $docObject->doc_id;
            $art_sort_position = $docObject->art_sort_position;
            $action = $docObject->action;


            $resulArticulos = self::getArticuloByDocumentsAndSortPosition($doc_id, $art_sort_position, $action);
            if ($resulArticulos == null) {
                return [
                    "status" => self::FAILED_CREATION,
                    "message" => "No hay resultados"
                ];
            } else {
                return [
                    "status" => self::SUCCESSFUL_CREATION,
                    "data" => self::encryptDataArticulo($resulArticulos)
                ];
            }
        }

    }

    private static function getAllById()
    {
        $body = file_get_contents('php://input');
        $docObject = json_decode($body);
        // Validar campos
        User::authorizationFromApp();

        if ($docObject !== null && ValidComponent::issetField($docObject->doc_id, 900)
            && ValidComponent::issetField($docObject->start_limit, 901)) {
            $start_limit = $docObject->start_limit;
            $doc_id = $docObject->doc_id;
            $resulArticulos = self::getArticulosByDocumID($doc_id, $start_limit);
            $concordancia = Correspondencia::encryptDataCorrespondencia(Correspondencia::getInfoCorrespondencia($doc_id));
            if ($start_limit > 0) {
                $concordancia = null;
            }
            return [
                "status" => self::SUCCESSFUL_CREATION,
                "rows" => count($resulArticulos),
                "total_rows" => self::getTotalRows($doc_id),
                "concordancia" => $concordancia,
                "data" => self::encryptDataArticulos($resulArticulos)
            ];
        } else {
            throw new ExceptionApi(self::INCORRECT_URL, "Parametros incorrectos", 400);
        }
    }

    private static function getArtById()
    {
        $body = file_get_contents('php://input');
        $docObject = json_decode($body);
        // Validar campos
        User::authorizationFromApp();

        if ($docObject !== null && ValidComponent::issetField($docObject->id_articulo, 100)) {
            $id_articulo = $docObject->id_articulo;
            $resulArticulos = self::getArticuloID($id_articulo);
            return [
                "status" => self::SUCCESSFUL_CREATION,
                "data" => self::encryptDataArticulo($resulArticulos)
            ];
        } else {
            throw new ExceptionApi(self::INCORRECT_URL, "Parametros incorrectos", 400);
        }

    }

    private static final function encryptDataArticulo($resulArticulos)
    {
        for ($i = 0; $i < count($resulArticulos); $i++) {
            $resulArticulos[$i]["id_articulo"] = PHP_AES_Cipher::encrypt(self::$encryption_key, $resulArticulos[$i]["id_articulo"]);
            $resulArticulos[$i]["doc_id"] = PHP_AES_Cipher::encrypt(self::$encryption_key, $resulArticulos[$i]["doc_id"]);
            $resulArticulos[$i]["idinterno"] = PHP_AES_Cipher::encrypt(self::$encryption_key, $resulArticulos[$i]["idinterno"]);
            $resulArticulos[$i]["art_desc"] = self::remove_non_used_code($resulArticulos[$i]["art_desc"]);
            //$resulArticulos[$i]["art_desc_fix"] = self::clean_article_actions(self::fix_html_string(self::remove_non_used_code($resulArticulos[$i]["art_desc"])));
            //$resulArticulos[$i]["art_desc_no_html"] = "";
            $resulArticulos[$i]["art_evolucion"] = "";
            $resulArticulos[$i]["art_evolucion_no_html"] = "";

        }

        return $resulArticulos;
    }

    public static final function encryptDataArticulos($resulArticulos)
    {
        for ($i = 0; $i < count($resulArticulos); $i++) {
            $resulArticulos[$i]["id_articulo"] = PHP_AES_Cipher::encrypt(self::$encryption_key, $resulArticulos[$i]["id_articulo"]);
            $resulArticulos[$i]["doc_id"] = PHP_AES_Cipher::encrypt(self::$encryption_key, $resulArticulos[$i]["doc_id"]);
            $resulArticulos[$i]["idinterno"] = PHP_AES_Cipher::encrypt(self::$encryption_key, $resulArticulos[$i]["idinterno"]);
            $resulArticulos[$i]["art_desc"] = self::remove_non_used_code($resulArticulos[$i]["art_desc"]);
            //$resulArticulos[$i]["art_desc_fix"] = self::clean_article_actions(self::fix_html_string(self::remove_non_used_code($resulArticulos[$i]["art_desc"])));
            //$resulArticulos[$i]["art_desc_no_html"] = "";
            $resulArticulos[$i]["art_evolucion"] = "";
            $resulArticulos[$i]["art_evolucion_no_html"] = "";
        }
        return $resulArticulos;
    }

    private static function getArticulosByDocumIDWithoutLimit($doc_id)
    {

        $query = "SELECT * FROM  " . self::TABLE_NAME . " WHERE " . self::DOC_ID . " = ? ORDER BY " . self::ART_SORT_POSITION . " ASC";
        $prepare_stm = ConnectionDB::getInstance()->getDataBase()->prepare($query);
        $prepare_stm->bindValue(1, $doc_id, PDO::PARAM_INT);
        if ($prepare_stm->execute()) {
            while ($row = $prepare_stm->fetch(PDO::FETCH_ASSOC)) {
                $rows[] = $row;
            }
            return $rows;
        } else {
            return null;
        }
    }

    private static function getArticulosByDocumID($doc_id, $start_limit)
    {

        $query = "SELECT * FROM  " . self::TABLE_NAME . " WHERE " . self::DOC_ID . " = ? ORDER BY " . self::ART_SORT_POSITION . " ASC LIMIT " . $start_limit . ",100;";
        $prepare_stm = ConnectionDB::getInstance()->getDataBase()->prepare($query);
        $prepare_stm->bindValue(1, $doc_id, PDO::PARAM_INT);
        if ($prepare_stm->execute()) {
            while ($row = $prepare_stm->fetch(PDO::FETCH_ASSOC)) {
                $rows[] = $row;
            }
            return $rows;
        } else {
            return null;
        }
    }

    private static function getArticuloID($id_articulo)
    {
        $query = "SELECT * FROM  " . self::TABLE_NAME . " WHERE " . self::ID_ARTICULO . " = ? ORDER BY " . self::ART_SORT_POSITION . " ASC;";
        $prepare_stm = ConnectionDB::getInstance()->getDataBase()->prepare($query);
        $prepare_stm->bindValue(1, $id_articulo, PDO::PARAM_INT);
        if ($prepare_stm->execute()) {
            while ($row = $prepare_stm->fetch(PDO::FETCH_ASSOC)) {
                $rows[] = $row;
            }
            return $rows;
        } else {
            return null;
        }
    }


    private static function getArticuloByDocumentsAndSortPosition($doc_id, $art_sort_position, $action)
    {

        if ($action == "NEXT") {
            $query = "SELECT * FROM " . self::TABLE_NAME . " WHERE " . self::DOC_ID . " = ? AND " . self::ART_SORT_POSITION . " >= ? order by art_sort_position ASC LIMIT 1;";
        } else if ($action == "PREVIOUS") {
            $query = "SELECT * FROM " . self::TABLE_NAME . " WHERE " . self::DOC_ID . " = ? AND " . self::ART_SORT_POSITION . " <= ? order by art_sort_position DESC LIMIT 1;";
        } else {
            $query = "SELECT * FROM " . self::TABLE_NAME . " WHERE " . self::DOC_ID . " = ? AND " . self::ART_SORT_POSITION . " = ?;";
        }

        $pdo = ConnectionDB::getInstance()->getDataBase();
        if ($art_sort_position == "0.0") {
            $query = "SELECT * FROM " . self::TABLE_NAME . " WHERE " . self::DOC_ID . " = ? ORDER BY " . self::ART_SORT_POSITION . " ASC LIMIT 1;";
            $prepare_stm = $pdo->prepare($query);
            $prepare_stm->bindValue(1, $doc_id, PDO::PARAM_INT);
        } else {
            $prepare_stm = $pdo->prepare($query);
            $prepare_stm->bindValue(1, $doc_id, PDO::PARAM_INT);
            $prepare_stm->bindValue(2, $art_sort_position, PDO::PARAM_STR);
        }

        if ($prepare_stm->execute()) {
            while ($row = $prepare_stm->fetch(PDO::FETCH_ASSOC)) {
                $rows[] = $row;
            }
            return $rows;
        } else {
            return null;
        }


    }

    public static function remove_non_used_code($texto_a_remover)
    {
        $array = array("* Ver Reforma", "Ver Jurisprudencia", "Ver Doctrina", "Ver Informaci&oacute;n Pr&aacute;ctica",
            "<a href=\"javascript:open_juris();\" class=\"lnkevo\"></a>",
            "<a href=\"javascript:open_doctrina();\" class=\"lnkevo\"></a>",
            "<a href=\"javascript:open_evolu();\" class=\"lnkevo\"></a>",
            "<strong></strong>", "<strong>&nbsp;</strong>",
            "<p style=\"text-align: justify; \"></p>", "<p></p>",
            "bgcolor=\"#ffffdd\" marginheight=\"10\" marginwidth=\"10\" topmargin=\"10\" leftmargin=\"10\" rightmargin=\"10\" bottommargin=\"10\"");

        $tmpReplacementOptions = array('<p><a href="javascript:open_infoprac(');
        foreach ($tmpReplacementOptions as $replacementItem) {
            $indexStart = stripos($texto_a_remover, $replacementItem);
            while ($indexStart !== false) {
                $endIndex = stripos($texto_a_remover, "</p>", $indexStart);
                $texto_a_remover = substr_replace($texto_a_remover, "", $indexStart, $endIndex - $indexStart);
                $indexStart = stripos($texto_a_remover, $replacementItem);
            }
        }

        foreach ($array as $item) {
            $texto_a_remover = str_ireplace($item, "", $texto_a_remover);
        }

        $arrayConcordancias = array("<p  style=\"text-align: justify; \" ><span ><b>Concordancias: </b>",
            "<p style=\"text-align: justify; \"><span><b>Concordancias: </b>",
            "<p style=\"text-align: justify;\"><span><b>Concordancias: </b>",
            "<p class=\"concorda\"  style=\"text-align: justify; \" ><span ><b>Concordancias: </b>");
        foreach ($arrayConcordancias as $item) {
            $texto_a_remover = str_ireplace($item, "<p  style=\"text-align: justify;word-wrap: break-word; \" ><span ><b>Concordancias: </b>", $texto_a_remover);
        }

        return $texto_a_remover;
    }

    private static function fix_html_string($original_text)
    {
        $original_text = self::format_article_content($original_text);
        $original_text = self::replace_spanish_chars($original_text);

        $tidy = new Tidy();
        $options = array(
            'show-body-only' => true
        );

        $tidy->parseString($original_text, $options);
        $tidy->cleanRepair();

        return $tidy->value;
    }

    private static function format_article_content($html_text)
    {
        $pos_body = stripos($html_text, "<body ");

        if ($pos_body === false) {
            $pos_body = stripos($html_text, "<body>");
        }

        if ($pos_body !== false) {
            $pos_body2 = strpos($html_text, ">", $pos_body);
            $html_text = substr($html_text, $pos_body2 + 1, strlen($html_text));
        }

        $html_text = str_ireplace("</body>", "", $html_text);

        return self::add_link_underline($html_text);
    }

    private static function replace_spanish_chars($original_text)
    {

        $original_text = str_replace("á", "&aacute;", $original_text);
        $original_text = str_replace("é", "&eacute;", $original_text);
        $original_text = str_replace("í", "&iacute;", $original_text);
        $original_text = str_replace("ó", "&oacute;", $original_text);
        $original_text = str_replace("ú", "&uacute;", $original_text);
        $original_text = str_replace("ñ", "&ntilde;", $original_text);
        $original_text = str_replace("ü", "&uuml;", $original_text);

        $original_text = str_replace("Á", "&Aacute;", $original_text);
        $original_text = str_replace("É", "&Eacute;", $original_text);
        $original_text = str_replace("Í", "&Iacute;", $original_text);
        $original_text = str_replace("Ó", "&Oacute;", $original_text);
        $original_text = str_replace("Ú", "&Uacute;", $original_text);
        $original_text = str_replace("Ñ", "&Ntilde;", $original_text);
        $original_text = str_replace("Ü", "&Uuml;", $original_text);

        return $original_text;

    }

    private static function add_link_underline($OriginalText)
    {

        return preg_replace('/(<a\b[^><]*)>/i', '$1 style="text-decoration:underline;">', $OriginalText);
    }

    private static function clean_article_actions($OriginalText)
    {

        return preg_replace("/\<a([^>]*)\>([^<]*)\<\/a\>/i", "$2", $OriginalText);
    }


    public static function getTotalRows($doc_id)
    {
        $query = "SELECT COUNT(" . self::ID_ARTICULO . ") as total FROM  " . self::TABLE_NAME . " WHERE " . self::DOC_ID . " = ? ORDER BY " . self::ID_ARTICULO . " ASC;";
        $prepare_stm = ConnectionDB::getInstance()->getDataBase()->prepare($query);
        $prepare_stm->bindValue(1, $doc_id, PDO::PARAM_INT);

        if ($prepare_stm->execute()) {
            $row = $prepare_stm->fetch(PDO::FETCH_ASSOC);
            return $row['total'];
        } else {
            return null;
        }
    }

}